@@ -95,38 +95,17 @@ Bool wcmDevSwitchModeCall(WacomDevicePtr priv, Bool absolute)
 	return TRUE;
 }
 
-
-static int wcmButtonPerNotch(WacomDevicePtr priv, int value, int threshold, int btn_positive, int btn_negative)
-{
-	int mode = is_absolute(priv);
-	int notches = value / threshold;
-	int button = (notches > 0) ? btn_positive : btn_negative;
-	int i;
-	WacomAxisData axes = {0};
-
-	for (i = 0; i < abs(notches); i++) {
-		wcmEmitButton(priv, mode, button, 1, &axes);
-		wcmEmitButton(priv, mode, button, 0, &axes);
-	}
-
-	return value % threshold;
-}
-
 static void wcmPanscroll(WacomDevicePtr priv, const WacomDeviceState *ds, int x, int y)
 {
 	WacomCommonPtr common = priv->common;
 	int threshold = common->wcmPanscrollThreshold;
-	int *accumulated_x, *accumulated_y;
 	int delta_x, delta_y;
 
 	if (!(priv->flags & SCROLLMODE_FLAG) || !(ds->buttons & 1))
 		return;
 
-	/* Tip has gone down down; store state for dragging */
+	/* Tip has gone down down; don't send pan event yet */
 	if (!(priv->oldState.buttons & 1)) {
-		priv->wcmPanscrollState = *ds;
-		priv->wcmPanscrollState.x = 0;
-		priv->wcmPanscrollState.y = 0;
 		return;
 	}
 
@@ -139,24 +118,20 @@ static void wcmPanscroll(WacomDevicePtr priv, const WacomDeviceState *ds, int x,
 		delta_y = (y - priv->oldState.y);
 	}
 
-	accumulated_x = &priv->wcmPanscrollState.x;
-	accumulated_y = &priv->wcmPanscrollState.y;
-	*accumulated_x += delta_x;
-	*accumulated_y += delta_y;
 
-	DBG(6, priv, "pan x = %d, pan y = %d\n", *accumulated_x, *accumulated_y);
+	DBG(6, priv, "pan x = %d, pan y = %d\n", delta_x, delta_y);
 
-	*accumulated_x = wcmButtonPerNotch(priv, *accumulated_x, threshold, 6, 7);
-	*accumulated_y = wcmButtonPerNotch(priv, *accumulated_y, threshold, 4, 5);
+	WacomAxisData axes = {0};
+	wcmAxisSet(&axes, WACOM_AXIS_SCROLL_X, -delta_x * PANSCROLL_INCREMENT/threshold);
+	wcmAxisSet(&axes, WACOM_AXIS_SCROLL_Y, -delta_y * PANSCROLL_INCREMENT/threshold);
+	wcmEmitMotion(priv, FALSE, &axes);
 }
 
 void wcmResetButtonAction(WacomDevicePtr priv, int button)
 {
 	WacomAction new_action = {};
 	int x11_button = priv->button_default[button];
-	char name[64];
 
-	sprintf(name, "Wacom button action %d", button);
 	wcmActionSet(&new_action, 0, AC_BUTTON | AC_KEYBTNPRESS | x11_button);
 	wcmActionCopy(&priv->key_actions[button], &new_action);
 }
@@ -164,9 +139,7 @@ void wcmResetButtonAction(WacomDevicePtr priv, int button)
 void wcmResetStripAction(WacomDevicePtr priv, int index)
 {
 	WacomAction new_action = {};
-	char name[64];
 
-	sprintf(name, "Wacom strip action %d", index);
 	wcmActionSet(&new_action, 0, AC_BUTTON | AC_KEYBTNPRESS | (priv->strip_default[index]));
 	wcmActionSet(&new_action, 1, AC_BUTTON | (priv->strip_default[index]));
 	wcmActionCopy(&priv->strip_actions[index], &new_action);
@@ -175,9 +148,7 @@ void wcmResetStripAction(WacomDevicePtr priv, int index)
 void wcmResetWheelAction(WacomDevicePtr priv, int index)
 {
 	WacomAction new_action = {};
-	char name[64];
 
-	sprintf(name, "Wacom wheel action %d", index);
 	wcmActionSet(&new_action, 0, AC_BUTTON | AC_KEYBTNPRESS | (priv->wheel_default[index]));
 	wcmActionSet(&new_action, 1, AC_BUTTON | (priv->wheel_default[index]));
 	wcmActionCopy(&priv->wheel_actions[index], &new_action);
